

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include "options.h"
#include "plink.h"

#define EPS  0.00001
#define EPS2 0.0000001

using namespace std;


void Plink::preCalcBins()
{

  // Determine, for this particular chromosome, how many bins we have
  // Now cM units will actually refer to bin numbers instead
  // Coding must be numeric per chromosome starting from 1,...
  
  // only use m1 to store a list of the bin numbers for this chromosome
  // use m2 to store the number of markers in that bin
  
  m1.resize(0);
  m2.resize(0);
  
  int b = (int)locus[par::run_start]->pos;
  
  m1.push_back(b);
  m2.push_back(1);
  
  for (int i=par::run_start;i<=par::run_end;i++)
    {
      if ((int)locus[i]->pos > b) 
	{
	  b = (int)locus[i]->pos;
	  m1.push_back(b);  // recode bin number
	  m2.push_back(1);  // start a new count
	  
	}
      else
	{
	  // one more marker for this first
	  m2[m2.size()-1]++;
	}
    }

//   // Report: 

//   cout << m1.size() << " bins\n";
//   for (int i=0; i<m1.size(); i++)
//     cout << m1[i] << " contains " 
// 	 << m2[i] << " markers\n";
  
}


vector<double> Plink::calcBins(vector<Z> & IBD, Z IBDg)
{
  vector<double> pihat;
  
  // Calculate average pihat for bin
  for (int b=0; b<m1.size(); b++)
    {
      Z binIBD;
      
      // Count out the number of markers in this bin
      for (int l=0; l<m2[b]; l++)
	{	  
	  binIBD.z0 += IBD[l].z0;
	  binIBD.z1 += IBD[l].z1;
	  binIBD.z2 += IBD[l].z2;
	}
      
      binIBD.z0 /= m2[b];
      binIBD.z1 /= m2[b];
      binIBD.z2 /= m2[b];
      
      //      pihat.push_back( (binIBD.z1*0.5+binIBD.z2) - (IBDg.z1*0.5 + IBDg.z2)  );
      pihat.push_back( (binIBD.z1*0.5+binIBD.z2) );
      
      if (par::verbose) 
	{
	  cout << "B " 
	       << pairid << " " 
	       << m1[b] << "\t"
	       << binIBD.z0 << " "
	       << binIBD.z1 << " "
	       << binIBD.z2 << " => "
	       << binIBD.z1*0.5+binIBD.z2 << " "
	       << (binIBD.z1*0.5+binIBD.z2) - (IBDg.z1*0.5 + IBDg.z2) << "\n";	  
	}
    }
  
  // Final analysis is genome-wide IBD
  if (!par::done_global_pihat) pihat_G.push_back( (IBDg.z1*0.5 + IBDg.z2) );

  return pihat;
}
