

//////////////////////////////////////////////////////////////////
//                                                              //
//           PLINK (c) 2005-2006 Shaun Purcell                  //
//                                                              //
// This file is distributed under the GNU General Public        //
// License, Version 2.  Please see the file COPYING for more    //
// details                                                      //
//                                                              //
//////////////////////////////////////////////////////////////////


#include <iostream>
#include <fstream>
#include <sstream>
#include "plink.h"
#include "options.h"


int Plink::readInformative()
{

  skip_pair.clear();

  //////////////////////////////////////////
  // Request to pre-read informative pairs?

  int c=0;
  
  printLOG("Reading pair inclusion data from [ "+par::inc_file+" ] \n");
  ifstream inp;
  inp.open(par::inc_file.c_str(), ifstream::in);
  inp.close();
  if(inp.fail())
    {
      inp.clear(ios::failbit);
      string msg = "No pair inclusion file [ " + par::inc_file + " ] exists.";
      error(msg);
    }
  
  ifstream INC;
  INC.open(par::inc_file.c_str());
  INC.clear();
  
  int l=0;

  for (int i1=0; i1<n-1; i1++)
    for (int i2=i1+1; i2<n; i2++)
      {

	if (INC.eof()) error("Problem with .inc file...");

	
	// Read status from file
	
	int v;
	INC >> v;
	
	if (v==1) //  Skip this pair in analysis
	  {
	    skip_pair.push_back(true);
	  }
	else if (v==2) // Include this pair
	  {
	    skip_pair.push_back(false);
	    c++;
	    
	    // Also need to calculate and save IBDg
	    Individual * p1 = sample[i1];
	    Individual * p2 = sample[i2];
	    
	    Z IBSg;
	    Z IBDg;
	    
	    // Calculate or fix
	    if (!par::FIXED)
	      {
		IBSg = calcGenomeIBS(p1,p2);
		IBDg = calcGenomeIBD(p1,p2,IBSg);
	      }
	    else
	      {
		IBDg = par::FIX_IBD;
	      }
	    
	    saved_IBDg.push_back(IBDg);

	}

      l++;

    }
  INC.clear();
  INC.close();

  return c;
}


int Plink::calcInformative() 
{
  //////////////////////////////
  // Precalculate to get list of 
  // individuals to skip in subsequent 
  // sections
  
  int c=0;
  int c0=0;

  printLOG("Preprocessing to assess number of informative pairs ... \n");
  for (int i1=0; i1<n-1; i1++)
    for (int i2=i1+1; i2<n; i2++)
      {	    

	c0++;

	Individual * p1 = sample[i1];
	Individual * p2 = sample[i2];

	if (!p1->missing && !p2->missing)
	  {
	    
	    Z IBSg;
	    Z IBDg;
	    
	    // Calculate or fix
	    if (!par::FIXED)
	      {
		IBSg = calcGenomeIBS(p1,p2);
		IBDg = calcGenomeIBD(p1,p2,IBSg);
	      }
	    else
	      {
		IBDg = par::FIX_IBD;
	      }
	    
	    	    
	    bool val = true;

	    //////////////////////////////
	    // Do we meet criteria?
	    
	    // Range of Genome-Wide IBD okay?
	    if (IBDg.z1/2 + IBDg.z2 < par::MIN_PIHAT ||
		IBDg.z1/2 + IBDg.z2 > par::MAX_PIHAT ) val = false;

	    // Not a parent-offspring pair?
	    if (IBDg.z1 > 0.95 ) val = false;
	    
	    // Affected-only pair analysis?
//	    if (par::affpair && (!isAffPair(p1,p2)) ) val = false;
	    if (par::affpair) 
	      error("Affected-pair analysis not yet fully implemented...\n");
	    
	    if (val)
	      {
		saved_IBDg.push_back(IBDg);
		skip_pair.push_back(false);
		++c;
		cout << c << " pairs extracted; "
		     << c0 << " processed of " << np 
		     << "            \r";
		cout.flush();
	      }
	    else
	      skip_pair.push_back(true);
	  }
	else
	  skip_pair.push_back(true);
      }		    
  
  
  stringstream s2;
  s2 << "\n" << c << " pairs are informative  ( " 
     << par::MIN_PIHAT 
     << " <= pihat <= " 
     << par::MAX_PIHAT        
     << " )\n";
  printLOG(s2.str());
  
  return c;
}


void Plink::writeInformative()
{
  //////////////////////////////////////////
  // Request to store informative pairs?

  printLOG("Writing pair inclusion data to [ " + par::output_file_name +".inc ] \n");
  ofstream INC;
  INC.open((par::output_file_name+".inc").c_str(), ifstream::out);
  for (int l=0; l<skip_pair.size(); l++)
    {
      if (skip_pair[l]) INC << "1\n";
      else INC << "2\n";
    }
  INC.close();
  
}
